@@ -30,7 +30,8 @@ import logging
 import signal
 
 sys.path.insert(0, os.path.join(os.path.dirname(__file__), '../'))
-from shadowsocks import utils, encrypt, eventloop, tcprelay, udprelay, asyncdns
+from shadowsocks import utils, daemon, encrypt, eventloop, tcprelay, udprelay,\
+    asyncdns
 
 
 def main():
@@ -38,6 +39,8 @@ def main():
 
     config = utils.get_config(False)
 
+    daemon.daemon_exec(config)
+
     utils.print_shadowsocks()
 
     if config['port_password']:
@@ -104,6 +107,7 @@ def main():
                     for pid in children:
                         try:
                             os.kill(pid, signum)
+                            os.waitpid(pid, 0)
                         except OSError:  # child may already exited
                             pass
                     sys.exit()
